;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3179efbbb476a7dab13cc6a016405745 . "./code.cg2js/cg-common-control.fasl")
(#x6823ead112de3f95d35bbfe8ff388529 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x37b4cad20b1ad7d42ad19a461f1d1a88 . "./code.cg2js/cg-yes-no-string.fasl")
(#x0cdada33824f943fa39b33960857d83b . "./code.cg2js/cg-clipboard.fasl")
(#xa44a7bc358c41b42e45c03f39232ed46 . "./code.cg2js/cg-dde-utils.fasl")
(#xc0049173d221e1c7f870bb0379545f7b . "./code.cg2js/cg-editable-text.fasl")
(#x1f4b37b0b491a52366c2170b5d13570c . "./code.cg2js/cg-player.fasl")
(#xdfbdec58dcbd3bb765324666ed37aaeb . "./code.cg2js/cg-text-or-combo.fasl")
(#x6c58f109dd907317ca6e021a2ec0c25f . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x1860e0946361162cb8470a78ad53346a . "./code.cg2js/cg-graphics-context.fasl")
(#xaae18c12517b2972c60a830dd38e8aa1 . "./code.cg2js/cg-choose-printer.fasl")
(#xf695e8259d031af9f6e1e6a2106a7bcd . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xe563d221987b2463e6f69c304004dc13 . "./code.cg2js/cg-edit-in-place.fasl")
(#x51a2231aa60714c0177575502154b4da . "./code.cg2js/cg-split-bar.fasl")
(#xee118c77c54a4d128757f01335fc1a29 . "./code.cg2js/cg-timer.fasl")
(#x5330bd8db8a13cba2fcdbb5ba9b54ed1 . "./code.cg2js/cg-progress-indicator.fasl")
(#x5019fa9e90f64718cde4bdae1bd541d6 . "./code.cg2js/cg-object-editor.fasl")
(#xc34198202cbee591e76a8457f1b68712 . "./code.cg2js/cg-fill-texture.fasl")
(#x955ccca6c46019c76aca0d7f6ea652f6 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xdee1ad169272f6ed8a8a8b406e01b1f1 . "./code.cg2js/cg-os-window.fasl")
(#xde207fb4872c399f0ec7a6af25d10c47 . "./code.cg2js/cg-color-dialog.fasl")
(#x417c534abaefb1decdfc9e1aabb02052 . "./code.cg2js/cg-combo-box.fasl")
(#x4dbf8ed2780c6928780b0e44aceb44b9 . "./code.cg2js/cg-directory-dialog.fasl")
(#xd4552ed157ba75c784c4df76052e2488 . "./code.cg2js/cg-tooltip.fasl")
(#xa66892f4e14b371b50f8a1bab07733c1 . "./code.cg2js/cg-lettered-menu.fasl")
(#xe82e923a1469294a90b789a6d1a78dd6 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x168b354806a6ebd16202f7fff25edb5a . "./code.cg2js/cg-pixmap.fasl")
(#x0747536c2d4e10f0fefaa61ad6befd31 . "./code.cg2js/cg.fasl")
(#x56514ee9f0d8aaa395708b167045e844 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x09d9e8ab21c44e334e5aeb5461043e5b . "./code.cg2js/cg-template-string.fasl")
(#x92eb7d45598ceee84e3b723f30ba20e0 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x3f61651c0bcb964fadcb4a12accafc41 . "./code.cg2js/cg-check-box.fasl")
(#x39bb756b2825ee8cd0178b646c71de26 . "./code.cg2js/cg-toggling-widget.fasl")
(#xc96de2ab236cead7ef6523898d84394c . "./code.cg2js/cg-radio-button.fasl")
(#x0704dc78ab457f0c90148bb1a9b1381d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xaa45e72d3712dc64ee40e387c2c439a3 . "./code.cg2js/cg-scaling-stream.fasl")
(#xa4e1720af8915a7d5e1ace9329faebe0 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x3ff76cb0e14d23588b1889793e4f45cd . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x48069c9231b21ce60802e296ffe9f748 . "./code.cg2js/cg-grid-widget.fasl")
(#xc6f9b2c26824854b3a302345a138533f . "./code.cg2js/cg-text-edit-pane.fasl")
(#xabab7d6ddce2758ec1fec2c760df4be0 . "./code.cg2js/cg-item-list.fasl")
(#x90ebb4dc198eeaaaa1932657c70ed3be . "./code.cg2js/cg-paren-matching.fasl")
(#xcf2bfd0ffe518f13a7cbfc4af92dcc4f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x874aa8adf1bf5463a96d9e38720b0d61 . "./code.cg2js/cg-html-dialog.fasl")
(#x5f2038c8d13d1c311b06d96ba118ff97 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3f8fa5ee42eb0b67769c243ac288e5b3 . "./code.cg2js/cg-project-window.fasl")
(#x35a24d8f7a37cd3f8fc258d8b4bcd819 . "./code.cg2js/cg-toolbar.fasl")
(#xeb6275959e4a7f3e7247aba55c572145 . "./code.cg2js/cg-caret.fasl")
(#xaa3ec3fb9dd77a02b2af426c7c2c313d . "./code.cg2js/cg-scroll-bar.fasl")
(#xfcd7c3c5f174984b8c09f1d76207e745 . "./code.cg2js/cg-web-browser.fasl")
(#xa0318574cf9e88af0906546373ad23b3 . "./code.cg2js/cg-dropping-outline.fasl")
(#xbf05f742426bb54367aa40f1c35f130b . "./code.cg2js/cg-font-dialog.fasl")
(#x97354a4d0086001ea9c745b8f14d75c9 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xea88577d5605e8f13212b9a9e31fba3b . "./code.cg2js/cg-yes-no-list.fasl")
(#xb4570241c969afe13e861173808f9e54 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x40e2233b1b73bfa75a9712c8da5c06d0 . "./code.cg2js/cg-lamp.fasl")
(#x0901c17520843a2eefb4d2510ebaf2a5 . "./code.cg2js/cg-drawable.fasl")
(#x78385f03014378630efd7684d5c21f77 . "./code.cg2js/cg-choice-list.fasl")
(#xa360c61c089c456062887e09f1ba3e6c . "./code.cg2js/cg-curve.fasl")
(#x9db562090bf233808b6ea866ae4ad7f6 . "./code.cg2js/cg-base.fasl")
(#xb0508ad924e0fe4b669a110a084b4d24 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x68f97f725bc2286af49ce58508ea6fff . "./code.cg2js/cg-printing.fasl")
(#x042458a614a40fd488c721d2db4f83bb . "./code.cg2js/cg-chart-or-plot.fasl")
(#xda62fab9261bb68bbeb320ade6cc0511 . "./code.cg2js/cg-hotspot.fasl")
(#x549e5d34ae00e9231d1fa6745efbcf57 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xb0ec056fa281a20e57fa5928f81f4758 . "./code.cg2js/cg-utility-dialog.fasl")
(#x71d87e7f2dfb28c350f2b9041d21524b . "./code.cg2js/cg-shortcut-menu.fasl")
(#x1407b41ce781cd7a400644f1b7f0f81e . "./code.cg2js/cg-comtab.fasl")
(#xd3243ec6e6e8d6110c323a1f5c2a53ff . "./code.cg2js/cg-os-widget.fasl")
(#x03fedf393333398ff74e07e09ad0ff20 . "./code.cg2js/cg-icon.fasl")
(#x903d4e5791b5ed191ffe43c21d0e683a . "./code.cg2js/cg-dev.fasl")
(#xa74446591d1a96e02c87213cd43719fd . "./code.cg2js/cg-calendar.fasl")
(#x177f1ece700c375831714162a53382e6 . "./code.cg2js/cg-group-box.fasl")
(#x069d272263e20367ba8c88f0d4264d08 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x57ed0157c443a469563301bd724f8b4c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x8060004fc872bd86c8c1801adb18ebb8 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb64aab1fca46f28593f55da673d06a1a . "./code.cg2js/cg-nodes.fasl")
(#x743f34b3a665005befa3f4f4eac20ab6 . "./code.cg2js/cg-intersect.fasl")
(#xac86ce5c13c757483fe4e94671adaedb . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x08a5fc1b73f6d0f26a4c643a8db04261 . "./code.cg2js/cg-class-support.fasl")
(#x656db16a6dfb32d76605761b121f9e6e . "./code.cg2js/cg-dialog-item.fasl")
(#xf004e78e8246cd614d08d0c441656fbe . "./code.cg2js/cg-plot-widget.fasl")
(#xfde7e9f96d6a60bff4ab67b4ad03835e . "./code.cg2js/cg-pixmap-printing.fasl")
(#x8eac57901a7d856cfd97cb424090dc31 . "./code.cg2js/cg-up-down-control.fasl")
(#x239bec2eceb2e397b81cf7895a45d4ef . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x31ad1714f714c75f458592101e85c6a8 . "./code.cg2js/cg-status-bar.fasl")
(#xdcdf3dbaf746f544e4b6aa4f8b514387 . "./code.cg2js/cg-class-grid.fasl")
(#x51679cfa7f6eeeff0f7a18b01bffd291 . "./code.cg2js/cg-wrap-string.fasl")
(#xd059a45cdd258306d35867a2b615572d . "./code.cg2js/cg-layout.fasl")
(#x468ba46c54a9055898f5785e1c843b0d . "./code.cg2js/cg-lisp-widget.fasl")
(#xf07614ceefd9763ab7f82679d87d3267 . "./code.cg2js/cg-button.fasl")
(#x160ae3f5f87017b57d40f59ac21521b1 . "./code.cg2js/cg-string-dialog.fasl")
(#xbaf513325a9ddfde389ce11e8abeca6c . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x0d13ec5165d5d7410b9ce316d71df9d1 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x4c035a352a8b1a7c25ca6d280e58125c . "./code.cg2js/cg-bitmap-stream.fasl")
(#xe0b905dfc237b2139d39099b2b0bc851 . "./code.cg2js/cg-static-text.fasl")
(#xf2bd4d22e4135c9f0c88ae5810fced8b . "./code.cg2js/cg-outline.fasl")
(#x696d7a2df072cc9dbfc819d3d9287942 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x2acda9044ab9d5f74374035a99fa9712 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x493f4a6f8d85c748dfb4f801e691b91d . "./code.cg2js/cg-picture-widget.fasl")
(#xbac52819c7ddc128c34927f64c8442ad . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x573704662f4d2c1ead294f6486420291 . "./code.cg2js/cg-chart-widget.fasl")
(#xacf5f8e417a142774fff855a7912213d . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x94bb972dbf490175c319307fdba2dc20 . "./code.cg2js/cg-lisp-text.fasl")
(#xef873be7a0edae3f9d3e3edd5dbd41d0 . "./code.cg2js/cg-palette.fasl")
(#x750ab72a1d70d7158e7b30a80f46cfe0 . "./code.cg2js/cg-property.fasl")
(#x087bb5d495c8a6f884ff955bbfa7444f . "./code.cg2js/cg-intersect-posbox.fasl")
(#x795bebb0606243bc4b4c82f8521bc390 . "./code.cg2js/cg-file-dialog.fasl")
(#x7f72c2159be684210cb955b25203ce0c . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x8ec068a6e7b82b1f7a603ca8fd77ae73 . "./code.cg2js/cg-get-pixmap.fasl")
(#x00cb0d19ba16d4b4da043760c79b07d5 . "./code.cg2js/cg-tray.fasl")
(#xfd8af0615464e8fc4dd7d57b8929cc7f . "./code.cg2js/cg-acache.fasl")
(#x5d60e270f42ff25fc2307113e72c3d92 . "./code.cg2js/cg-message-dialog.fasl")
(#x1055ca61a32de65fb5c29a71c3a76c15 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x5990e29eab35e1969fee02116a5423de . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xd02290ec07a367d73f428e5f8f35e1d6 . "./code.cg2js/cg-tab-control.fasl")
(#x69d0fde7d08bcb759a874e547145ebf5 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xf831dbeb2d434866d5e68f42a352ed65 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x7da79f618c8b66b05669c848759492c8 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x113252ad99e349594de71ad49ffde956 . "./code.cg2js/cg-menu.fasl")
(#xb7134042535355110f49c935ecb3b2b0 . "./code.cg2js/cg-get-position.fasl")
(#x39e646696036f2cd914aaa65371758c4 . "./code.cg2js/cg-trackbar.fasl")
(#xdae4594afe6286fc5d98d7843bdf7729 . "./code.cg2js/cg-selected-object.fasl")
(#x83e2bd1ce00bfbc222584a752c3ff1ee . "./code.cg2js/cg-list-view.fasl")
(#x79b57fa48f23aa9518ab81b4cd04f924 . "./code.cg2js/cg-html-widget.fasl")
(#x61d56a6d5556f0c2ea9bdcea62ca9d18 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x48f4f3990c220b7fa36b86b755d3ae76 . "./code.cg2js/cg-text-widget.fasl")
(#x00a7dd4cffa03e7383216c6c60c9295f . "./code.idejs/ide-base.fasl")
(#xd1d77399c58d5bdd08f34b4e96d6c894 . "./code.idejs/ide-process-browser.fasl")
(#x67f9c10c59aea1cbb2f5b2a16905812f . "./code.idejs/ide-patch-dialog.fasl")
(#x1090e698b2b2f44c9195192b2a86ecf8 . "./code.idejs/ide-keyshort.fasl")
(#xf337d6584a35b394d4be236c9718f471 . "./code.idejs/ide-window-list.fasl")
(#x95c7a95b561f98489d21e20c43c3b126 . "./code.idejs/ide.fasl")
(#xc1a1ba419b70135c21fb7e606bbb3418 . "./code.idejs/ide-editor.fasl")
(#x6ac657fb39b799cec614dc5b67e3139d . "./code.idejs/ide-clipboard.fasl")
(#xb5f5e715c7bdbb4e8d0df37890af2b9a . "./code.idejs/ide-trace.fasl")
(#xbd72c5a8e3488a4982fbdaab780a8359 . "./code.idejs/ide-class-browser.fasl")
(#xb5f0ef0a082c7e5c4b3f4a9099834112 . "./code.idejs/ide-grapher.fasl")
(#x5d74f714fec7a967b405679f646ecee1 . "./code.idejs/ide-apropos.fasl")
(#x8ee2845e505b452b648d8536ec181b68 . "./code.idejs/ide-navigator.fasl")
(#xf0eee5e061f75ea9f4a18bfe3ec8571b . "./code.idejs/ide-find-definitions.fasl")
(#x1f7a0f27caceec7ef0214edb225114dc . "./code.idejs/ide-project.fasl")
(#x15f8e0da4ade00f11007aa08f6d5acaa . "./code.idejs/ide-class-grapher.fasl")
(#x826ff0c71ab0fb21619d678541783d6d . "./code.idejs/ide-acache.fasl")
(#x02ce5cee02aabf5311cc5ddc4d1d976e . "./code.idejs/ide-find-in-files.fasl")
(#x9933a6f035c7af7f59d553f18dab6167 . "./code.idejs/ide-inspector.fasl")
(#x6a9fc155d6c53264b8bb5ae2bfd6a133 . "./code.idejs/ide-intro.fasl")
(#x2ad4302196148fcbbe809bab954c4f12 . "./code.idejs/ide-package-list.fasl")
(#xc5143234a37823b3258a30d23e1b3963 . "./code.idejs/ide-dev.fasl")
(#x1fb1bfc5b220c078ace59060c2e6082b . "./code.idejs/ide-form.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
